#!/bin/bash

# ***************************************************************************************
#     Pleora Technologies Inc. Copyright (c) 2002-2018
# ***************************************************************************************

SCRIPT_DIR=`readlink -f $0`
SDK_BIN_DIR=`dirname $SCRIPT_DIR`
SDK_DIR=`cd $SDK_BIN_DIR/..; pwd`
SDK_LIB_DIR="$SDK_DIR/lib"
USER_ID=`id -u`

SERVICE_NAME="eBUSd"
SERVICE_FILE="/etc/init.d/$SERVICE_NAME"
DAEMON_NAME="eBUSd"
DAEMON_FILE="$SDK_BIN_DIR/$DAEMON_NAME"
SERVICE_PID_FILE="/var/run/$SERVICE_NAME.pid"

Install()
{
    cp $SDK_DIR/etc/init.d/$SERVICE_NAME $SERVICE_FILE
    chmod u=rwx,g=rwx,o=rx $SERVICE_FILE
}

Uninstall()
{
    if [ -f $SERVICE_FILE ]; then
        echo "Removing $SERVICE_FILE."
        rm -f $SERVICE_FILE
    fi
}

Register()
{
    echo "Configuring $SERVICE_NAME for automatic startup."

    if which update-rc.d >/dev/null 2>&1; then
        update-rc.d -f $1 defaults
    fi
}

Unregister()
{
    if [ -f $SERVICE_FILE ]; then

        RESULT=`pgrep $1`
        if [ ! -z "$RESULT" ]; then
            echo "eBUS daemon is running: stopping the service."
            service $1 stop
        fi

        echo "Disabling automatic startup of $1."

        if which update-rc.d >/dev/null 2>&1; then
            update-rc.d -f $1 remove 1> /dev/null
        elif which chkconfig >/dev/null 2>&1; then
            RESULT=`chkconfig --list 2>/dev/null | grep $1`
            if [ ! -z "$RESULT" ]; then
                chkconfig --del $1
            fi
        fi
    fi
}

INSTALL="no"
UNINSTALL="no"
DISTRIBUTION="Unknown"

# Parse the input arguments
for i in $*
do
    case $i in
        --install)
            INSTALL="auto"
            ;;
        --uninstall)
            UNINSTALL="yes"
            ;;
        *)
        echo "avalable options are: --install or --uninstall"
        exit 1
        ;;
    esac
done

# Validate the entry
if [ "$UNINSTALL" = "yes" ]; then
    INSTALL="skip"
else
    if [ "$INSTALL" != "no" ]; then
        if [ ! which update-rc.d >/dev/null 2>&1 ]; then
            echo "WARNING: Cannot install the eBUS daemon for automatic startup because the "
            echo " update-rc.d application is not present in this distribution."
        fi
    else
        echo "Unknown eBUS daemon installation mode."
    fi
fi

if [ "$INSTALL" != "no" ]; then

    DISTRIBUTION="not set"
    if [ -f /etc/lsb-release ]; then
        if [ -f /lib/lsb/init-functions ]; then
            DISTRIBUTION=`grep DISTRIB_ID /etc/lsb-release | sed -e 's/DISTRIB_ID=//'`
        fi
    elif [ -f /etc/os-release ]; then
        if [ -f /lib/lsb/init-functions ]; then
            DISTRIBUTION=`grep -w NAME /etc/os-release | sed 's/[^"]*"\([^"]*\)"/\1/'`
            if [ "$DISTRIBUTION" == "Debian GNU/Linux" ] ; then
                # Debian is close enough for the purposes of this script
                DISTRIBUTION="Ubuntu"
            fi
        fi
    fi

    if [ "$DISTRIBUTION" != "Ubuntu" ]; then
        echo "Unsupported distribution. Cannot generate the file format."
        exit 3
    fi
fi

# Check required priviledge
if [ "$USER_ID" != "0" ]; then
  echo "You need to run this script as superuser (root account)."
  exit 1
fi

# Now perform the actions...

# Always uninstall the stuff if we can to present weird states...
Unregister $SERVICE_NAME $DISTRIBUTION
Uninstall $SERVICE_NAME $SERVICE_LINK

# The system is clean, now we can try to install if we have to do it!
if [ "$INSTALL" != "skip" ]; then
    Install $DISTRIBUTION
    if [ "$INSTALL" = "auto" ]; then    
        Register $SERVICE_NAME $DISTRIBUTION
        echo "Starting eBUS daemon."
        service $SERVICE_NAME start
    else
        echo "The eBUS daemon can now be managed manually."
        echo " Example: "
        echo "    Start the eBUS daemon with the following command:"
        echo "         service $SERVICE_NAME start"
        echo "    Stop the eBUS daemon with the following command:"
        echo "         service $SERVICE_NAME stop"
        echo "    Query the status of the eBUS daemon with the following command:"
        echo "         service $SERVICE_NAME status"
    fi

    # Add rule for Pleora U3V devices all the time if udev available
    if [ -d "/etc/udev/rules.d" ]; then
        pushd $SDK_BIN_DIR >> /dev/null
        ./set_udev_rules.sh --add=28b7
        ./set_udev_rules.sh --add=14fb
        popd >> /dev/null
    fi

else
    pushd $SDK_BIN_DIR >> /dev/null
    ./set_udev_rules.sh --remove=28b7
    ./set_udev_rules.sh --remove=14fb
    popd >> /dev/null
fi
